@@ -6,7 +6,7 @@
 ;;         Protesilaos Stavrou <info@protesilaos.com>
 ;; Maintainer: Theme-Buffet Development <~bboal/general-issues@lists.sr.ht>
 ;; URL: https://git.sr.ht/~bboal/theme-buffet
-;; Version: 0.1.0
+;; Version: 0.1.1
 ;; Package-Requires: ((emacs "29.1"))
 
 ;; This file is part of GNU Emacs.
@@ -51,15 +51,16 @@
 ;;
 ;; The binding above will set the themes to be either Modus or Ef, authored by
 ;; Protesilaos Stavrou <https://git.sr.ht/~protesilaos>, distributed across six
-;; periods of the day (night, twilight, morning, day, afternoon and evening).  The
-;; library will not require the aforementioned package, you will have to install it
-;; manual if you intent to use it.  Finally to start using Theme-Buffet, evaluate:
+;; periods of the day (night, twilight, morning, day, afternoon and evening).
+;; The library will not require the aforementioned package, you will have to
+;; install it manual if you intent to use it.  Finally to start using
+;; Theme-Buffet, evaluate:
 ;;
 ;;    (theme-buffet-mode 1)
 ;;
 ;; Following the appanage way of Emacs, both the names and number of themes and
-;; time periods can be freely changed while mantaining the same structure.  There
-;; is also a time-offset that can be set by the user to match a specific
+;; time periods can be freely changed while mantaining the same structure.
+;; There is also a time-offset that can be set by the user to match a specific
 ;; time-zone/personal preference.  E.g
 ;;
 ;;    (setq theme-buffet-time-offset 2)
@@ -92,15 +93,15 @@
 ;;    (theme-buffet-timer-mins 30) ; to change theme every 30m from now
 ;;    (theme-buffet-timer-hours 2) ; to also change every 2h from now
 ;;
-;; Interactively, as an example, you would press M-x and execute
+;; Interactively, as an example, you would press M-x
 ;; `theme-buffet-order-other-period'.  Then, after choosing any defined period,
-;;you would get returned a random loaded theme from the aforementioned period.
+;; you would get returned a random loaded theme from the aforementioned period.
 
 ;;
 ;; Disclaimer from Bruno Boal to the reader: This package was produced during my
-;; learning sessions with Protesilaos "Prot" Stavrou and improved as
-;; homework.  Most of the credit goes to him, the mistakes you may find are my
-;; own.  Personally, despite the disadvantages and advantages of not being a
+;; learning sessions with Protesilaos "Prot" Stavrou and improved as homework.
+;; Most of the credit goes to him, the mistakes you may find are my own.
+;; Personally, despite the disadvantages and advantages of not being a
 ;; professional programmer, it is essential for me to always have fun and
 ;; enjoyment during learning and programming.  In this respect, mission
 ;; accomplished, a big "thank you!" to my mentor.  Also, keep in mind at least
@@ -297,7 +298,7 @@ information."
     (or (remq (car custom-enabled-themes) theme-list)
         theme-list)))
 
-(defun theme-buffet--load-random (period)
+(defun theme-buffet--load-random (&optional period)
   "Load random theme according to PERIOD.
 
 Omit current theme if it's not the only pertaining to the list of the
@@ -305,12 +306,13 @@ corresponding period.  Being this the case, the same theme shall be served.
 
 An error message will appear if the theme is not available to load through
 `load-theme'."
-  (if-let ((themes (theme-buffet--get-theme-list period))
-           (chosen-theme (seq-random-elt themes))
-           ((memq chosen-theme (custom-available-themes))))
-      (theme-buffet--reload-theme chosen-theme)
-    (user-error "Theme-Buffet Chef says `%s' is not known or installed!"
-                chosen-theme)))
+  (let ((period (or period (theme-buffet--get-period-keyword))))
+    (if-let ((themes (theme-buffet--get-theme-list period))
+             (chosen-theme (seq-random-elt themes))
+             ((memq chosen-theme (custom-available-themes))))
+        (theme-buffet--reload-theme chosen-theme)
+      (user-error "Theme-Buffet Chef says `%s' is not known or installed!"
+                  chosen-theme))))
 
 (defvar theme-buffet-theme-history nil
   "Theme-Buffet period history.")
@@ -329,7 +331,8 @@ An error message will appear if the theme is not available to load through
   (declare (interactive-only t))
   (interactive)
   (let ((chosen-theme (intern (theme-buffet--theme-prompt))))
-    (theme-buffet--reload-theme chosen-theme "according to your wishes. Enjoy..." )))
+    (theme-buffet--reload-theme chosen-theme
+                                "according to your wishes. Enjoy..." )))
 
 (defvar theme-buffet-period-history nil
   "Theme-Buffet period history.")
@@ -372,13 +375,71 @@ An error message will appear if the theme is not available to load through
 (defvar theme-buffet-timer-mins nil
   "Timer that calls another Theme-Buffet's Sous-Chef into the kitchen.")
 
-(defun theme-buffet--free-timer (timer-obj)
-  "Cancel and set to nil the timer TIMER-OBJ."
+(defun theme-buffet--free-timer (timer-obj &optional no-message)
+  "Cancel and set to nil the timer TIMER-OBJ.
+With optional NO-MESSAGE, does not notify the user."
   (when-let (((boundp timer-obj))
              (obj (symbol-value timer-obj)))
     (cancel-timer obj)
     (set timer-obj nil)
-    (message "Break time in the Theme-Buffet kitchen!")))
+    (unless no-message
+      (message "Break time in the Theme-Buffet kitchen!"))))
+
+(defun theme-buffet-free-all-timers ()
+  "Give a break to Theme-Buffet staff.
+All timer variables and functions are canceled."
+  (interactive)
+  (cancel-function-timers #'theme-buffet--load-random)
+  (mapc (lambda (timer)
+          (theme-buffet--free-timer timer :no-message))
+        '(theme-buffet-timer-mins
+          theme-buffet-timer-hours
+          theme-buffet-timer-periods)))
+
+(defun theme-buffet--active-timers ()
+  "Get list of strings with the suffix of the active timers.
+E.g If both the periods and mins timers are active, the returned list is as
+  follows: (\"periods\" \"mins\")"
+  (let* ((var-len (length "theme-buffet-timer-"))
+         (active-timers
+          (mapcar (lambda (timer)
+                    (if (symbol-value timer)
+                        (substring (symbol-name timer) var-len)))
+                  '(theme-buffet-timer-periods
+                    theme-buffet-timer-mins
+                    theme-buffet-timer-hours))))
+    (delq nil active-timers)))
+
+(defun theme-buffet-clear-timers ()
+  "Check active timers and prompt the user to choose which to clear."
+  (declare (interactive-only t))
+  (interactive)
+  (if-let ((prompt "Choose a timer to clear/cancel: ")
+           (collection (theme-buffet--active-timers))
+           (choice (completing-read prompt collection nil t)))
+      (cond
+       ((string-equal choice "periods")
+        (theme-buffet--free-timer 'theme-buffet-timer-periods))
+       ((string-equal choice "mins")
+        (theme-buffet--free-timer 'theme-buffet-timer-mins))
+       ((string-equal choice "hours")
+        (theme-buffet--free-timer 'theme-buffet-timer-hours))
+       (t
+        (user-error "Invalid choice in `theme-buffet-clear-timers'")))
+    (user-error "You didn't send a single Chef into the kitchen")))
+
+
+;;;###autoload
+(define-minor-mode theme-buffet-mode
+  "Theme-Buffet serves your preferred themes according to the time of day.
+You eyes will thank you.  Or not...
+
+The preference for the themes is specified in the `theme-buffet-menu'"
+  :global t
+  (if theme-buffet-mode
+      (unless (plistp (theme-buffet--selected-menu))
+           (user-error "`theme-buffet-menu' isn't passing the health inspections as it is!"))
+    (theme-buffet-free-all-timers)))
 
 
 (defmacro theme-buffet--define-timer (units)
@@ -391,22 +452,30 @@ naming."
       ('mins (setq factor 60 max-num 180))
       ('hours (setq factor 3600 max-num 12))
       (_ (user-error
-          "Bad `units' arg on `theme-buffet--define-timer %s'" units)))
+          "Wrong arg on `theme-buffet--define-timer': %s" units)))
     `(defun ,fn-name (number)
        ,(format "Set interactively the timer for NUMBER of %s.
 When NUMBER is 0, the timer is cancelled. Maximum value is %s" units max-num)
        (interactive
-        (list (read-number ,(format "Theme Buffet service in how many %s? " units) nil
-                           'theme-buffet-user-timers-history)))
+        (list
+         (read-number
+          ,(format "Theme Buffet service in how many %s? (0 to cancel) " units)
+          nil 'theme-buffet-user-timers-history)))
+       (or theme-buffet-mode (theme-buffet-mode 1))
        (if-let (((natnump number))
                 ((<= number ,max-num))
-                (timer-secs (* ,factor number)))
-           (if (equal number 0)
+                (timer-secs (* ,factor number))
+                (msg-1 "Theme-Buffet Sous-Chef is")
+                (msg-2 "rushing into the kitchen..."))
+           (if (= number 0)
                (theme-buffet--free-timer ',fn-name)
+             (setq msg-2 (if ,fn-name
+                             "waiting for the requisition"
+                           msg-2))
+             (theme-buffet--free-timer ',fn-name :no-message)
              (setq ,fn-name (run-at-time timer-secs timer-secs
-                                         #'theme-buffet--load-random
-                                         (theme-buffet--get-period-keyword)))
-             (message "Theme-Buffet Sous-Chef is rushing into the kitchen..."))
+                                         #'theme-buffet--load-random))
+             (message "%s %s" msg-1 msg-2))
          (user-error "The input number should be a natural up to %s instead of `%s'"
                      ,max-num number)))))
 
@@ -433,12 +502,12 @@ opinion.")
     `(defun ,(intern (format "theme-buffet-%s" menu)) ()
        ,docstring
        (interactive)
+       (or theme-buffet-mode (theme-buffet-mode 1))
        (theme-buffet--free-timer 'theme-buffet-timer-periods)
        (setq theme-buffet-menu (quote ,menu)
              theme-buffet-timer-periods
              (run-at-time t (theme-buffet--interval)
-                          #'theme-buffet--load-random
-                          (theme-buffet--get-period-keyword)))
+                          #'theme-buffet--load-random))
        (message "Sucess! Theme-Buffet Chef is firing up %s themes..." ',menu))))
 
 ;;;###autoload (autoload 'theme-buffet-built-in "theme-buffet")
@@ -449,18 +518,6 @@ opinion.")
 (theme-buffet--define-menu-defuns end-user)  ; (theme-buffet-end-user)
 
 
-;;;###autoload
-(define-minor-mode theme-buffet-mode
-  "Theme-Buffet serves your preferred themes according to the time of day.
-You eyes will thank you.  Or not...
-
-The preference for the themes is specified in the `theme-buffet-menu'"
-  :global t
-  (if theme-buffet-mode
-      (unless (plistp (theme-buffet--selected-menu))
-           (user-error "`theme-buffet-menu' isn't passing the health inspections as it is!"))
-    (cancel-function-timers #'theme-buffet--load-random)))
-
 
 (provide 'theme-buffet)
 ;;; theme-buffet.el ends here
